{
**********ARTIFICIAL BEE COLONY ALGORITHM FOR CONSTRAINED OPTIMIZATION*********************

A Modified Artificial Bee Colony (ABC) Algorithm for Constrained Optimization Problems

Dervis Karaboga, Bahriye Akay
Contact: karaboga@erciyes.edu.tr, bahriye@erciyes.edu.tr

Please refer to the papers for detailed information

D. Karaboga, B. Akay, A Modified Artificial Bee Colony (ABC) Algorithm for Constrained Optimization Problems Applied Soft Computing, 11 (3), 3021-3031, 2011. 
B. Akay, D. Karaboga, Artificial Bee Colony Algorithm for Large-Scale Problems and Engineering Design Optimization, Journal of Intelligent Manufacturing, In Press, doi:10.1007/s10845-010-0393-4
D. Karaboga, B. Basturk, Artificial Bee Colony (ABC) Optimization Algorithm for Solving Constrained Optimization Problems, LNCS: Advances in Soft Computing: Foundations of Fuzzy Logic and Soft Computing, Vol: 4529/2007, pp: 789-798, Springer- Verlag, 2007. 

Please define the cost function and the constraints of your problem in the file "functions.pas" 
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Spin,math, Series, TeEngine, TeeProcs, Chart,functions;



type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    Label2: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    popsize: TSpinEdit;
    MG: TSpinEdit;
    run: TSpinEdit;
    fun: TComboBox;
    Button1: TButton;
    TLimit: TEdit;
    Label9: TLabel;
    Sonuckutu: TMemo;
    Label11: TLabel;
    Label3: TLabel;
    Label7: TLabel;
    TPR: TEdit;
    Label5: TLabel;
    Tspp: TEdit;
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;


var
Form1: TForm1;

x,u:Tmatris;
bestx: Tvektor;


Cycle,maxCycle,NumberofRuns:integer;
limit,MR,spp:real;

f,fu,fitu,fit,violation,violationu: Tvektor;
prob,trial:TVektor;

scoutindex,scoutCounter:integer;
maxTrial:real;

bestviolation:real;

//variables for statistical purpose
bestindex,worstindex:integer;
min,bestfit,minsofruns:Tvektor;
maxim,sum,sum1,sum2,bestmaxim:real;
stdDev,minf,minAvg:real;



epsilon:real; //All equality constraints are converted into inequality constraints, abs(hj)<=epsilon
tol:real; //tol is used to determine whether a solution is feasible



Procedure SetParams();
procedure WriteObtained();



implementation


{$R *.dfm}
Procedure SetParams();
begin

d:=GetDimension(form1.fun.ItemIndex); //d:The number of the parameters of the optimization problem
//d parameter is assigned automatically according to the function selected from combobox
//Please change the lines of the GetDimension function according to the number of the parameters of your cost function


np:=Form1.popsize.Value div 2;   //np: the number of food sources
maxCycle:=form1.MG.Value;   // maxCycle: maximum number oc cycles
NumberofRuns:=form1.run.Value;   //NumberofRuns: to allow the algorithm run multiple times for statistical purposes
MR:=strtofloat(form1.TPR.Text); //MR: Modification rate
limit:=round(np*d)*10; //limit: the control parameter "limit".
//limit:=strtofloat(form1.TLimit.Text);  //Either np*d*10 or another value can be set.
form1.TLimit.Text:=floattostr(limit);
spp:=round(np*d);//strtofloat(form1.Tspp.text);  //spp: scout production period
tol:=0.000000000001;


SetLength(hi,d+1);       //hi: a vector storing the upper bounds of the parameters
SetLength(lo,d+1);       //lo: a vector storing the lower bounds of the parameters

//the values of the elements in lo and hi vectors (boundaries of the paramaters)
// are assigned in GetBounds function depending on the function chosen from the combobox
//Please change the lines of the GetBounds  function according to your cost function parameters
GetBounds(form1.fun.ItemIndex);


SetLength(bestx,d+1);    //bestx: global best vector

SetLength(f,np+1);       //f: a vector storing the cost function values of the current population
SetLength(fit,np+1);     //fit: a vector storing the fitness values of the current population
SetLength(fu,np+1);      //fu: a vector storing the cost function values of the neighbour solutions
SetLength(fitu,np+1);    //fitu: a vector storing the cost function values of the neighbour solutions

SetLength(violation,np+1);    //violation: a vector storing the contraint violations of the current population
SetLength(violationu,np+1);   //violationu:a vector storing the contraint violations of the neighbour solutions


SetLength(prob,np+1);     //prob: a vector storing the probability values proportional to the fitness and the contraint violatios
SetLength(trial,np+1);    //trial: a vector storing the number of local searches in the neighbourhood of each solution

Setlength(x,np+1,d+1);     // x: current population with the size of NP*D
Setlength(u,np+1,d+1);     // new solutions produced in the neighbourhood of the current solutions are stored in "u"



SetLength(min,maxCycle+1);    //Min vector stores the minimum cost function of each cycle
SetLength(bestfit,maxCycle+1); //BestFit vector stores thebest fitness value of each cycle
setlength(minsofruns,NumberofRuns+1);   //minsofruns: a vector storing the Global Minimum of each run



end;


procedure initial1;
var
i,j:integer;
begin
 {In the first step, ABC algorithm generates a randomly distributed initial population of np
solutions (food source positions), Each solution ~xi (i = 1; 2; :::; np) is a D-dimensional vector.

ABC algorithm does not consider the initial population to be feasible.

In initialization phase, random values between the lower and the
upper boundaries of the parameters are assigned for the parameters of solutions.}

minf:=0;
bestviolation:=exp(40);
scoutCounter:=0;



  for i:=1 to np do
  begin
  fit[i]:=0;
  f[i]:=0;
  violation[i]:=exp(10);
  trial[i]:=0;
    for j:=1 to d do
    begin
    x[i,j]:=(lo[j]+random*(hi[j]-lo[j]));
    {
    values are truncated for the parameters to be integer}


    if form1.fun.ItemIndex=1 then
    begin
    x[i,1]:=trunc(x[i,1]/0.0625)*0.0625;
    x[i,2]:=trunc(x[i,2]/0.0625)*0.0625;
    end;
    if form1.fun.ItemIndex=3 then
    begin
    x[i,1]:=trunc(x[i,1]);
    x[i,2]:=trunc(x[i,2]);
    x[i,3]:=trunc(x[i,3]);
    x[i,4]:=trunc(x[i,4]);
    end;

      if form1.fun.ItemIndex=4 then
    begin
    x[i,3]:=trunc(x[i,3]);

    end;
    u[i,j]:=x[i,j];
    end;
  end;

end;

procedure OffSpring(i:integer);   //Generate a new solution in the neighbourhood of ith solution.
var
j,r1,param:integer;

begin
{In the unconstrained version of ABC algorithm, only one randomly chosen parameter xij
is modified while other parameters except for xij are copied from the old solution xi. However,
in the modified ABC algorithm, for each parameter xij, a uniformly distributed random real number, (
0  Rj  1), is produced and if the real number is less than the control parameter
value, modification rate (MR), and the parameter xij is modified. }
   repeat
      r1:=trunc(random*np+1);
    until (r1<>i);

    param:= trunc(random*D+1);
   for j:=1 to d do
       if random<MR then
          u[i,j]:=x[i,j]+(x[i,j]-x[r1,j])*(0.5-random)*2
      else
        u[i,j]:=x[i,j];

        //Here, we ensure at least one parameter to be modified.
        u[i,param]:=x[i,param]+(x[i,param]-x[r1,param])*(0.5-random)*2  ;

            {values are truncated for the parameters to be integer. If your problem does not have such a restriction, remove these lines}
      if form1.fun.itemindex=1 then
      begin
        u[i,1]:=(trunc(u[i,1]/0.0625))*0.0625;
        u[i,2]:=(trunc(u[i,2]/0.0625))*0.0625;
      end;

      if form1.fun.itemindex=3 then
      begin
        u[i,1]:=trunc(u[i,1]);
        u[i,2]:=trunc(u[i,2]);
        u[i,3]:=trunc(u[i,3]);
        u[i,4]:=trunc(u[i,4]);
      end;

      if form1.fun.itemindex=4 then
      begin
        u[i,3]:=trunc(u[i,3]);
      end;


      {If a parameter value produced by this operation
exceeds its predetermined boundaries, the parameter can be set to an acceptable value. In this
work, the value of the parameter exceeding its boundary is set to its boundaries.}
      for j:=1 to d do
      begin
        if u[i,j]<lo[j] then u[i,j]:=lo[j];
        if u[i,j]>hi[j] then u[i,j]:=hi[j];
      end;


    // Evaluate the ith solution of the population in terms of cost function, fitness function and constrain violation
    fu[i]:=CalculateFunction(u,i,form1.fun.ItemIndex);
    fitu[i]:=calculateFitness(fu[i]);
    violationu[i]:=Calculatepenalty(u,i,form1.fun.ItemIndex);

end;

procedure evaluatePop();
{
--------
Evaluate all of the solutions in the population in terms of cost function,
fitness function and constrain violation
--------
}
var
i:integer;
begin
for i:=1 to np do
begin

f[i]:=CalculateFunction(x,i,form1.fun.ItemIndex);
    fit[i]:=calculateFitness(f[i]);
    violation[i]:=Calculatepenalty(x,i,form1.fun.ItemIndex);
    end;
end;

procedure select(i:integer);
var
j:integer;
begin
{
After producing a new food source (solution), ABC algorithm makes a selection. The second
modification of ABC algorithm to solve constrained optimization problems is using Debs rules
instead of using greedy selection in selection process.

Another handling mechanism may be inserted here to handle with constraints.

Structure of the algorithm directs the solutions to feasible region in running process due to the Debs rules employed instead of greedy
selection. Applying Debs rules, the bee either memorizes the new position by forgetting the
old one or keeps the old one. Debs method uses a tournament selection operator, where two
solutions are compared at a time by applying following criteria:

# Among two feasible solutions(violation[i] <= 0, violation[j] <= 0), the one having better objective
function value is preferred ( f[i] < f[j], solution i is dominant),
}

    if (violation[i]<=tol) and (violationu[i]<=tol)  then      //both feasible
    begin
      if fitu[i]>fit[i] then
        begin
          trial[i]:=0;
          fit[i]:=fitu[i];
          f[i]:=fu[i];
          violation[i]:=violationu[i];
            for j:=1 to d do
              x[i,j]:=u[i,j];
        end
        else
        trial[i]:=trial[i]+1;
    end
    {# Any feasible solution (violation[i] <= 0) is preferred to any infeasible solution
(violation[j] > 0) (solution i is dominant),}
    else if (violation[i]>tol) and (violationu[i]<=tol)  then      //new feasible
        begin
          trial[i]:=0;
          fit[i]:=fitu[i];
          f[i]:=fu[i];
          violation[i]:=violationu[i];
            for j:=1 to d do
              x[i,j]:=u[i,j];
        end
    else
        {# Among two infeasible solutions (violation[i] > 0, violation[j] > 0), the one having smaller
constraint violation is preferred (violation[i] < violation[j], solution i is dominant).}
    begin
    trial[i]:=trial[i]+1;
    if (violationu[i]>tol) and (violation[i]>tol) then   //both infeasible
      begin
        if violationu[i]<violation[i] then
          begin
          //trial[i]:=0;
          violation[i]:=violationu[i];
          f[i]:=fu[i];
          fit[i]:=fitu[i];//0.000000001;
            for j:=1 to d do
              x[i,j]:=u[i,j];
          end;
      end;
    end;
 

end;

procedure EmployedBee;
var
i:integer;
begin
{An employed bee produces a modification on the position of the food source, evaluates the nectar
amount (fitness value, quality) of the new source (new solution). and selects the better one.}
  for i:=1 to np do
  begin
    Offspring(i);
    select(i);
  end;
end;



procedure best;
var
i,j,index:integer;
begin
maxim:=fit[1];
index:=1;
    //Selecting the best solution is achived based on Deb's Rules as in Selection process.
for i:=1 to np do
begin
if (violation[i]<=tol) and (violation[index]<=tol) then
  begin
  if maxim<fit[i] then
    begin
    maxim:=fit[i];
    index:=i;
    end;
  end
else
if (violation[i]<=tol) and (violation[index]>tol) then
  begin
  maxim:=fit[i];
  index:=i;
  end
else
if (violation[i]>tol) and (violation[index]>tol) then
  begin
  if violation[i]<violation[index] then
    begin
    maxim:=fit[i];
    index:=i;
    end;
  end;
  //if violation[i]>0 and violation[index]=0 then no change
end;


if (bestviolation<=tol) and (violation[index]<=tol) then      //both feasible
begin
  if bestmaxim<maxim then
  begin
    bestmaxim := maxim;
    minf:=f[index];
    bestviolation := violation[index];
    for j:=1 to d do
      bestx[j]:=x[index,j];
  end;
end
else
if (bestviolation>tol) and (violation[index]>tol) then    //both infeasible
begin
  if violation[index]<bestviolation then
    bestmaxim:=maxim;
    minf:=f[index];
    bestviolation:=violation[index];
    for j:=1 to d do
      bestx[j]:=x[index,j];
end
else
if (bestviolation>tol) and (violation[index]<=tol) then  //global infeasible current feasible
begin
  bestmaxim:=maxim;
  minf:=f[index];
  bestviolation:=violation[index];
    for j:=1 to d do
    bestx[j]:=x[index,j];
end;


end;




procedure OnlookerBee;
var
s,t:integer;
sumfit,sumviolation:real;
begin
sumfit:=0;
sumviolation:=0;
{After all employed bees complete the search process, they share the nectar information of
the food sources and their position information with the onlooker bees on the dance area by
calculating probability values.}

  for s:=1 to np do
    begin
      if violation[s]<=tol then
        sumfit:=sumfit+(fit[s]);
        sumviolation:=sumviolation+violation[s];
    end;

    sumfit:=sumfit+0.000000001;
    sumviolation:=sumviolation+0.000000001;
    

{Since infeasible solutions are allowed to populate in the colony, a third modification is needed
here to assign probability values for infeasible solutions as well as for feasible ones. Probability
values of infeasible solutions are between 0 and 0.5 while those of feasible ones are between
0.5 and 1. Please refer to the papers above for the complete equations. Another assignment formula
may also be used here.}
for s:=1 to np do
  begin
    if violation[s]<=tol then
      prob[s]:=0.5+(fit[s]/sumfit)*0.5
    else
      prob[s]:=(1-(violation[s]/sumviolation))*0.5;
  end;

{By a selection mechanism like roulette wheel, solutions are selected probabilistically
proportional to their fitness values incase of feasible solutions and inversely proportional to their
violation values incase of infeasible solutions. Another selection mechanism may also be used here. }
t:=0;
s:=1;
repeat
    if random<prob[s] then
      begin
        t:=t+1;
        Offspring(s);
        {An onlooker bee evaluates the nectar information taken from all employed bees and chooses
a food source with a probability related to its nectar amount. As in the case of the employed bee,
onlooker bee produces a modification on the position of the food source selected based on the
probability value and checks its nectar amount.}
        select(s);
    end;
    s:=s+1;
    if s=np+1 then s:=1;
until(t=np);
end;

procedure scout;
var
i,j:integer;
begin
{After all onlookers are distributed, food sources that are not worth to exploit anymore are determined.}

scoutCounter:=scoutCounter+1;
maxTrial:=trial[1];
scoutindex:=1;


 {Solution that has maximum trial numbers is determined}
for i:=1 to np do
begin
if (maxTrial<trial[i]) then
begin
maxTrial:=trial[i];
scoutindex:=i;
end;
end;


{If a solution can not be improved through a predetermined number of cycles (limit),
it is decided to be abandoned.}


{ Another difference between the versions of ABC proposed for unconstrained and constrained problems is that
artificial scouts are produced at a predetermined period of cycles for discovering new food sources randomly.
This period is another control parameter of the modified algorithm called scout production period (SPP). At each SPP cycle,
it is controlled if there is an abandoned food source exceeding limit or not.
}
if (maxTrial>limit) and (scoutCounter>spp) then
begin
  scoutCounter:=0;
   for j:=1 to d do
        {The food source abandoned by the bees is replaced with a new food source discovered by the scouts.
        This is simulated by producing a position randomly and replacing it with the abandoned one.}
        u[scoutindex,j]:=(lo[j]+random*(hi[j]-lo[j]));

 {values are truncated for the parameters to be integer}
 if form1.fun.itemindex=1 then
 begin
  u[scoutindex,1]:=trunc(u[scoutindex,1]/0.0625)*0.0625;
  u[scoutindex,2]:=trunc(u[scoutindex,2]/0.0625)*0.0625;
 end;

 if form1.fun.itemindex=3 then
 begin
  u[scoutindex,1]:=trunc(u[scoutindex,1]);
  u[scoutindex,2]:=trunc(u[scoutindex,2]);
  u[scoutindex,3]:=trunc(u[scoutindex,3]);
  u[scoutindex,4]:=trunc(u[scoutindex,4]);
 end;

 if form1.fun.ItemIndex=4 then
 begin
    u[scoutindex,3]:=trunc(u[scoutindex,3]);
 end;
  trial[scoutindex]:=0;
  fu[scoutindex]:=calculatefunction(u,scoutindex,form1.fun.ItemIndex);
  violationu[scoutindex]:=calculatepenalty(u,scoutindex,form1.fun.ItemIndex);
  fitu[scoutindex]:=calculatefitness(fu[scoutindex]);


  //replacement
  for j:=1 to d do
    x[scoutindex,j]:=u[scoutindex,j];
    fit[scoutindex]:=fitu[scoutindex];
    violation[scoutindex]:=violationu[scoutindex];
end


end;




procedure TForm1.Button1Click(Sender: TObject);
var
i,ks:integer;
begin
SetParams();
Form1.Sonuckutu.Lines.Clear;


for i:=1 to maxCycle do
begin


bestfit[i]:=0;
min[i]:=0;
end;

MinAvg:=0;
stdDev:=0;

for ks:=1 to NumberofRuns do
begin
  randomize;
  Cycle:=0;
  bestmaxim:=0.000000001;
  initial1;
  best;
  REPEAT
    Cycle:=Cycle+1;
    employedBee;
    OnlookerBee;
    best;
    scout;
    min[Cycle]:=min[Cycle]+minf;
    bestfit[Cycle]:=bestfit[Cycle]+bestmaxim;
   UNTIL (Cycle=maxCycle);
   Form1.Sonuckutu.lines.Add(inttostr(ks)+'.Run of '+inttostr(NumberofRuns)+' Runs') ;
    WriteObtained();
    MinAvg:=MinAvg+(minf);
    minsofruns[ks]:=minf;
    end;

MinAvg:=MinAvg/NumberofRuns;
bestindex:=1;
worstindex:=1;

  for ks:=1 to NumberofRuns do
    begin
    stdDev:=stdDev+sqr(minsofruns[ks]-MinAvg);
    if minsofruns[ks]<minsofruns[bestindex] then bestindex:=ks;
    if minsofruns[ks]>minsofruns[worstindex] then worstindex:=ks;
    end;
    stdDev:=sqrt(stdDev/(NumberofRuns));

 form1.sonuckutu.lines.add('');
 Form1.sonuckutu.Lines.add(' Best of:  '+inttostr(NumberofRuns)+' runs: '+floattostr(minsofruns[bestindex]));
 Form1.sonuckutu.Lines.add(' Means of: '+inttostr(NumberofRuns)+' runs: '+floattostr(MinAvg));
 Form1.sonuckutu.Lines.add(' Worst of: '+inttostr(NumberofRuns)+' runs: '+floattostr(minsofruns[worstindex]));
 Form1.sonuckutu.Lines.add(' Std.Dev. of: '+inttostr(NumberofRuns)+' runs: '+floattostr(stdDev));
 form1.sonuckutu.lines.add('');


 sonuckutu.Lines.SaveToFile(form1.fun.Items.Strings[form1.fun.itemindex]+'.txt');

end;



procedure TForm1.Button3Click(Sender: TObject);
begin
form1.Button1.Enabled:=True;
end;

procedure WriteObtained();
{----------
This function does not calculate the cost function or contraint violations of a solution or the popualtion.
This was written in order to print the cost function and constraints of the Global Best Solution.
Please write your functions in functions.pas
-----------
}



var
i:integer;
var
u,v,w,snc:real;
c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11:real;
P,L,E,G,J,M,tomax,sigmamax,deltamax,delta,sigma,Pc,to3,to1,to2,R:real;
Dc,N,k,wt,dic,sigmaak,alfa:real;
begin
{bestx[1]:=3.499999;//-1.717143;
bestx[2]:=0.7;//1.595709;
bestx[3]:=17;//1.827247;
bestx[4]:=7.30;//-0.7636413;
bestx[5]:=7.8 ;//-0.763645;
bestx[6]:=3.350215 ;
bestx[7]:= 5.286683;
}


for i:=1 to d do
Form1.sonuckutu.Lines.add('x['+IntToStr(i)+']:'+floattostr(bestx[i]));
form1.sonuckutu.Lines.add('');


if form1.fun.ItemIndex=0 then
begin
{bestx[1]:=0.20573;
bestx[2]:=3.47049;
bestx[3]:=9.03662;
bestx[4]:=0.20573;
}
P:=6000;
L:=14;
E:=30000000;
G:=12000000;
tomax:=13600;
sigmamax:=30000;
deltamax:=0.25;

R:=sqrt((sqr(bestx[2])/4)+sqr((bestx[1]+bestx[3])/2));
J:=2*((sqrt(2)*bestx[1]*bestx[2])*((sqr(bestx[2])/12)+(sqr((bestx[1]+bestx[3])/2))));
M:=P*(L+0.5*bestx[2]);
to1:=P/(sqrt(2)*bestx[1]*bestx[2]);
to2:=(M*R)/J;
to3:=sqrt(sqr(to1)+((2*to1*to2*bestx[2])/(2*R))+sqr(to2));
sigma:=(6*P*L)/(bestx[4]*sqr(bestx[3]));
delta:=4*P*power(L,3)/(E*power(bestx[3],3)*bestx[4]);
Pc:=(4.013*E*sqrt(sqr(bestx[3])*power(bestx[4],6)/36)/sqr(L))*(1-(bestx[3]/(2*L)*sqrt(E/(4*G))));

c1:=to3-tomax;
c2:=sigma-sigmamax;
c3:=bestx[1]-bestx[4];
c4:=0.10471*sqr(bestx[1])+0.04811*bestx[3]*bestx[4]*(14+bestx[2])-5;
c5:=0.125-bestx[1];
c6:=delta-deltamax;
c7:=P-Pc;

snc:=1.10471*sqr(bestx[1])*bestx[2]+0.04811*bestx[3]*bestx[4]*(14+bestx[2]);

form1.sonuckutu.lines.add('g1 :'+floattostr(c1));
form1.sonuckutu.lines.add('g2 :'+floattostr(c2));
form1.sonuckutu.lines.add('g3 :'+floattostr(c3));
form1.sonuckutu.lines.add('g4 :'+floattostr(c4));
form1.sonuckutu.lines.add('g5 :'+floattostr(c5));
form1.sonuckutu.lines.add('g6 :'+floattostr(c6));
form1.sonuckutu.lines.add('g7 :'+floattostr(c7));
form1.sonuckutu.lines.add('Cost :'+floattostr(snc));
end;

if form1.fun.ItemIndex=1 then
begin
{bestx[1]:=0.8125;
bestx[2]:=0.4375;
bestx[3]:=42.09845;
bestx[4]:=176.6366;
 }
snc:=0.6224*bestx[1]*bestx[3]*bestx[4]+1.7781*bestx[2]*sqr(bestx[3])+3.1661*sqr(bestx[1])*bestx[4]+19.84*sqr(bestx[1])*bestx[3];
c1:=-bestx[1]+0.0193*bestx[3];
c2:=-bestx[2]+0.00954*bestx[3];
c3:=-pi*sqr(bestx[3])*bestx[4]-((4/3)*pi*power(bestx[3],3))+1296000;
c4:=bestx[4]-240;
form1.sonuckutu.lines.add('g1 :'+floattostr(c1));
form1.sonuckutu.lines.add('g2 :'+floattostr(c2));
form1.sonuckutu.lines.add('g3 :'+floattostr(c3));
form1.sonuckutu.lines.add('g4 :'+floattostr(c4));
form1.sonuckutu.lines.add('Cost :'+floattostr(snc));
end;

if form1.fun.ItemIndex=2 then
begin
c1:=1-((power(bestx[2],3)*bestx[3])/(71785*power(bestx[1],4)));
c2:=((4*sqr(bestx[2])-bestx[1]*bestx[2])/(12566*(power(bestx[1],3)*(bestx[2]-power(bestx[1],4)))))+(1/(5108*sqr(bestx[1])))-1;
c3:=1-((140.45*bestx[1])/((bestx[3]*sqr(bestx[2]))));
c4:=((bestx[1]+bestx[2])/1.5)-1;
snc:=sqr(bestx[1])*bestx[2]*(bestx[3]+2);
form1.sonuckutu.lines.add('g1 :'+floattostr(c1));
form1.sonuckutu.lines.add('g2 :'+floattostr(c2));
form1.sonuckutu.lines.add('g3 :'+floattostr(c3));
form1.sonuckutu.lines.add('g4 :'+floattostr(c4));
form1.sonuckutu.lines.add('Cost :'+floattostr(snc));
end;

if form1.fun.ItemIndex=4 then
begin
c1:=(27/(bestx[1]*sqr(bestx[2])*bestx[3]))-1;
c2:=(397.5/(bestx[1]*sqr(bestx[2])*sqr(bestx[3])))-1;
c3:=((1.93*power(bestx[4],3))/(bestx[2]*bestx[3]*power(bestx[6],4)))-1;
c4:=((1.93*power(bestx[5],3))/(bestx[2]*bestx[3]*power(bestx[7],4)))-1;
c5:=(sqrt(sqr((745*bestx[4])/(bestx[2]*bestx[3]))+(16.9*power(10,6)))/(110*power(bestx[6],3)))-1;
c6:=(sqrt(sqr((745*bestx[5])/(bestx[2]*bestx[3]))+(157.7*power(10,6)))/(85*power(bestx[7],3)))-1;
c7:=(bestx[2]*bestx[3]/40)-1;
c8:=(5*bestx[2]/bestx[1])-1;
c9:=(bestx[1]/(12*bestx[2]))-1;
c10:=((1.5*bestx[6]+1.9)/bestx[4])-1;
c11:=((1.1*bestx[7]+1.9)/bestx[5])-1;

snc:=(0.7854*bestx[1]*sqr(bestx[2])*(3.3333*sqr(bestx[3])+14.9334*bestx[3]-43.0934))-(1.508*bestx[1]*(sqr(bestx[6])+sqr(bestx[7])))+(7.4777*(power(bestx[6],3)+power(bestx[7],3)))+(0.7854*(bestx[4]*sqr(bestx[6])+bestx[5]*sqr(bestx[7])));

form1.sonuckutu.lines.add('g1 :'+floattostr(c1));
form1.sonuckutu.lines.add('g2 :'+floattostr(c2));
form1.sonuckutu.lines.add('g3 :'+floattostr(c3));
form1.sonuckutu.lines.add('g4 :'+floattostr(c4));
form1.sonuckutu.lines.add('g5 :'+floattostr(c5));
form1.sonuckutu.lines.add('g6 :'+floattostr(c6));
form1.sonuckutu.lines.add('g7 :'+floattostr(c7));
form1.sonuckutu.lines.add('g8 :'+floattostr(c8));
form1.sonuckutu.lines.add('g9 :'+floattostr(c9));
form1.sonuckutu.lines.add('g10 :'+floattostr(c10));
form1.sonuckutu.lines.add('g11 :'+floattostr(c11));

form1.sonuckutu.lines.add('Cost :'+floattostr(snc));
end;
form1.sonuckutu.Lines.add('-------------');

end;


end.
