unit functions;

interface
uses Math,SysUtils, Dialogs;

type
TMatris=array of array of real;
TVektor=array of real;

var
d,np:integer;
hi,lo: Tvektor;
epsilon:real; //All equality constraints are converted into inequality constraints, abs(hj)<=epsilon





function weldedbeam(pop:Tmatris;i:integer):real;
function cpweldedbeam(pop:Tmatris;i:integer):real;

function pressureVessel(pop:Tmatris;i:integer):real;
function cppressureVessel(pop:Tmatris;i:integer):real;

function tensionCompression(pop:Tmatris;i:integer):real;
function cptensionCompression(pop:Tmatris;i:integer):real;

function geartrain(pop:Tmatris;i:integer):real;
function cpgeartrain(pop:Tmatris;i:integer):real;


function  cpSpeedReducer(pop:Tmatris;i:integer):real;
function  SpeedReducer(pop:Tmatris;i:integer):real;

procedure GetBounds(index:integer);

function CalculateFunction(pop:Tmatris;i,index:integer):real;
function calculateFitness(f:real):real;
function CalculatePenalty(pop:Tmatris;i,index:integer):real;
function GetDimension(index:integer):integer;

implementation


function GetDimension(index:integer):integer;
begin
case index of
0: result:=4;
1: result:=4;
2: result:=3;
3: result:=4;
4: result:=7;
end;
end;
function calculateFitness(f:real):real;
begin
 if f<=0 then
   result:=1+abs(f)
  else
    result:=1/(1+abs(f));
end;

function CalculatePenalty(pop:Tmatris;i,index:integer):real;
begin
case index of
0: result:=cpweldedbeam(pop,i);
1: result:=cppressurevessel(pop,i);
2: result:=cptensioncompression(pop,i);
3: result:=cpgeartrain(pop,i) ;
4: result:=cpspeedReducer(pop,i);
end;
end;

function CalculateFunction(pop:Tmatris;i,index:integer):real;
begin
case index of
0: result:=weldedbeam(pop,i);
1: result:=pressurevessel(pop,i);
2: result:=tensioncompression(pop,i);
3: result:=geartrain(pop,i) ;
4: result:=speedReducer(pop,i);
end;
end;//funcandfit



function weldedbeam(pop:Tmatris;i:integer):real;
begin
result:=1.10471*sqr(pop[i,1])*pop[i,2]+0.04811*pop[i,3]*pop[i,4]*(14+pop[i,2])
end;

function cpweldedbeam(pop:Tmatris;i:integer):real;
var
P,L,E,G,J,M,tomax,sigmamax,deltamax,delta,sigma,Pc,to3,to1,to2,R:real;
c1,c2,c3,c4,c5,c6,c7:real;
begin
P:=6000;
L:=14;
E:=30000000;
G:=12000000;
tomax:=13600;
sigmamax:=30000;
deltamax:=0.25;

R:=sqrt((sqr(pop[i,2])/4)+sqr((pop[i,1]+pop[i,3])/2));
J:=2*((sqrt(2)*pop[i,1]*pop[i,2])*((sqr(pop[i,2])/12)+(sqr((pop[i,1]+pop[i,3])/2))));
M:=P*(L+0.5*pop[i,2]);
to1:=P/(sqrt(2)*pop[i,1]*pop[i,2]);
to2:=(M*R)/J;
to3:=sqrt(sqr(to1)+((2*to1*to2*pop[i,2])/(2*R))+sqr(to2));
sigma:=(6*P*L)/(pop[i,4]*sqr(pop[i,3]));
delta:=4*P*power(L,3)/(E*power(pop[i,3],3)*pop[i,4]);
Pc:=(4.013*E*sqrt(sqr(pop[i,3])*power(pop[i,4],6)/36)/sqr(L))*(1-(pop[i,3]/(2*L)*sqrt(E/(4*G))));

c1:=to3-tomax;
c2:=sigma-sigmamax;
c3:=pop[i,1]-pop[i,4];
c4:=0.10471*sqr(pop[i,1])+0.04811*pop[i,3]*pop[i,4]*(14+pop[i,2])-5;
c5:=0.125-pop[i,1];
c6:=delta-deltamax;
c7:=P-Pc;

result:=0;
if c1>0 then result:=result+c1;
if c2>0 then result:=result+c2;
if c3>0 then result:=result+c3;
if c4>0 then result:=result+c4;
if c5>0 then result:=result+c5;
if c6>0 then result:=result+c6;
if c7>0 then result:=result+c7;

end;

function pressureVessel(pop:Tmatris;i:integer):real;
begin
result:=0.6224*pop[i,1]*pop[i,3]*pop[i,4]+1.7781*pop[i,2]*sqr(pop[i,3])+3.1661*sqr(pop[i,1])*pop[i,4]+19.84*sqr(pop[i,1])*pop[i,3];
end;

function cppressureVessel(pop:Tmatris;i:integer):real;
var
c1,c2,c3,c4:real;
begin
result:=0;
c1:=-pop[i,1]+0.0193*pop[i,3];
c2:=-pop[i,2]+0.00954*pop[i,3];
c3:=-pi*sqr(pop[i,3])*pop[i,4]-((4/3)*pi*power(pop[i,3],3))+1296000;
c4:=pop[i,4]-240;
if c1>0 then result:=result+c1;
if c2>0 then result:=result+c2;
if c3>0 then result:=result+c3;
if c4>0 then result:=result+c4;
end;

function tensionCompression(pop:Tmatris;i:integer):real;
begin
result:=sqr(pop[i,1])*pop[i,2]*(pop[i,3]+2);
end;

function cptensionCompression(pop:Tmatris;i:integer):real;
var
c1,c2,c3,c4:real;
begin
c1:=1-((power(pop[i,2],3)*pop[i,3])/(71785*power(pop[i,1],4)));
c2:=((4*sqr(pop[i,2])-pop[i,1]*pop[i,2])/(12566*(power(pop[i,1],3)*(pop[i,2]-power(pop[i,1],4)))))+(1/(5108*sqr(pop[i,1])))-1;
c3:=1-((140.45*pop[i,1])/((pop[i,3]*sqr(pop[i,2]))));
c4:=((pop[i,1]+pop[i,2])/1.5)-1;
result:=0;
if c1>0 then result:=result+c1;
if c2>0 then result:=result+c2;
if c3>0 then result:=result+c3;
if c4>0 then result:=result+c4;

end;

function geartrain(pop:Tmatris;i:integer):real;
begin
result:=sqr((1/6.931)-((pop[i,3]*pop[i,2])/(pop[i,1]*pop[i,4])))
end;

function cpgeartrain(pop:Tmatris;i:integer):real;
begin
result:=0;
end;



function  cpSpeedReducer(pop:Tmatris;i:integer):real;
var
c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11:real;
begin
c1:=(27/(pop[i,1]*sqr(pop[i,2])*pop[i,3]))-1;
c2:=(397.5/(pop[i,1]*sqr(pop[i,2])*sqr(pop[i,3])))-1;
c3:=((1.93*power(pop[i,4],3))/(pop[i,2]*pop[i,3]*power(pop[i,6],4)))-1;
c4:=((1.93*power(pop[i,5],3))/(pop[i,2]*pop[i,3]*power(pop[i,7],4)))-1;
c5:=(sqrt(sqr((745*pop[i,4])/(pop[i,2]*pop[i,3]))+(16.9*power(10,6)))/(110*power(pop[i,6],3)))-1;
c6:=(sqrt(sqr((745*pop[i,5])/(pop[i,2]*pop[i,3]))+(157.7*power(10,6)))/(85*power(pop[i,7],3)))-1;
c7:=(pop[i,2]*pop[i,3]/40)-1;
c8:=(5*pop[i,2]/pop[i,1])-1;
c9:=(pop[i,1]/(12*pop[i,2]))-1;
c10:=((1.5*pop[i,6]+1.9)/pop[i,4])-1;
c11:=((1.1*pop[i,7]+1.9)/pop[i,5])-1;


result:=0;
if c1>0 then result:=result+c1;
if c2>0 then result:=result+c2;
if c3>0 then result:=result+c3;
if c4>0 then result:=result+c4;
if c5>0 then result:=result+c5;
if c6>0 then result:=result+c6;
if c7>0 then result:=result+c7;
if c8>0 then result:=result+c8;
if c9>0 then result:=result+c9;
if c10>0 then result:=result+c10;
if c11>0 then result:=result+c11;
end;

function  SpeedReducer(pop:Tmatris;i:integer):real;
begin
result:=(0.7854*pop[i,1]*sqr(pop[i,2])*(3.3333*sqr(pop[i,3])+14.9334*pop[i,3]-43.0934))-(1.508*pop[i,1]*(sqr(pop[i,6])+sqr(pop[i,7])))+(7.4777*(power(pop[i,6],3)+power(pop[i,7],3)))+(0.7854*(pop[i,4]*sqr(pop[i,6])+pop[i,5]*sqr(pop[i,7])));
end;

procedure GetBounds(index:integer);
begin
case index of

0:
begin
  lo[1]:=0.1;hi[1]:=2;
  lo[2]:=0.1;hi[2]:=10;
  lo[3]:=0.1;hi[3]:=10;
  lo[4]:=0.1;hi[4]:=2;
end;
1:
begin
  lo[1]:=0;hi[1]:=99;
  lo[2]:=0;hi[2]:=99;
  lo[3]:=10;hi[3]:=200;
  lo[4]:=10;hi[4]:=200;
end;
2:
begin
  lo[1]:=0.05;hi[1]:=2;
  lo[2]:=0.25;hi[2]:=1.3;
  lo[3]:=2;hi[3]:=115;
end;
3:
begin
  epsilon:=0;
  lo[1]:=12;hi[1]:=60;
  lo[2]:=12;hi[2]:=60;
  lo[3]:=12;hi[3]:=60;
  lo[4]:=12;hi[4]:=60;
end;

4:
begin
  epsilon:=0.001;
  lo[1]:=2.6;hi[1]:=3.6;
  lo[2]:=0.7;hi[2]:=0.8;
  lo[3]:=17;hi[3]:=28;
  lo[4]:=7.3;hi[4]:=8.3;
  lo[5]:=7.8;hi[5]:=8.3;
  lo[6]:=2.9;hi[6]:=3.9;
  lo[7]:=5;hi[7]:=5.5;

end;

end;//case
end;








end.
